from OpenGL import GL
import pygame

import screen
import client
import level
import sprite
import data

def char_range(first,last):
	return "".join(map(chr,(range(ord(first),ord(last)+1))))
allowed_letters = ".-" + char_range('0','9') + char_range('a','z') + char_range('A','Z')

class ClientScreen (screen.Screen):
	def __init__(self):
		super(ClientScreen,self).__init__()
		self.client = None
		self.server_hostname = ""
		self.label = None
		self.server_label = None
		self.error_label = sprite.SpriteImage.getTextSpriteImage(
				"Could not connect to the server", font_size = 24)
		self.show_error = False
		self._update_labels()
		self.button_sprite = sprite.SpriteImage.getSpriteImage(data.filepath("button.png"))
		self.connect_text = sprite.SpriteImage.getTextSpriteImage(
				"Connect", font_size = 24)
	
	def _connect(self):
		self.show_error = False
		try:
			self.client = client.Client(self.server_hostname)
		except:
			self.show_error = True
		self._update_labels()
	
	def _quit(self):
		self.next_level = menu_screen.MenuScreen()
	
	def _update_labels(self):
		self.server_label = sprite.SpriteImage.getTextSpriteImage(
			"Server: %s" % self.server_hostname, font_size = 24)
		if self.client != None:
			self.label = sprite.SpriteImage.getTextSpriteImage(
				"Waiting for server to start the game..", font_size = 24)
		else:
			self.label = sprite.SpriteImage.getTextSpriteImage(
				"Enter the server hostname or IP", font_size = 24)
	
	def draw(self):
		GL.glTranslatef(-400,-300,0)
		sprite.set_color((1,1,1,1))
		
		GL.glPushMatrix()
		GL.glTranslatef((800-self.label.size[0])/2,480,0)
		self.label.draw()
		GL.glPopMatrix()
		
		if self.client == None:
			GL.glPushMatrix()
			GL.glTranslatef((800-self.button_sprite.size[0])/2,220,0)
			self.button_sprite.draw()
			GL.glPopMatrix()
			GL.glPushMatrix()
			offset = (self.button_sprite.size[1]-self.connect_text.size[1])/2
			GL.glTranslatef((800-self.connect_text.size[0])/2,220+offset,0)
			self.connect_text.draw()
			GL.glPopMatrix()
		
		GL.glPushMatrix()
		GL.glTranslatef(150,120,0)
		self.server_label.draw()
		GL.glPopMatrix()
		
		if self.show_error:
			GL.glPushMatrix()
			GL.glTranslatef(150,380,0)
			self.error_label.draw()
			GL.glPopMatrix()
	
	def do_frame(self, time_passed):
		if self.client != None:
			try:
				to_start = self.client.check_start_game()
			except Exception, e:
				print "Server disconnected before starting the game"
				self.client = None
				return
			if to_start:
				self.next_level = level.Level(None, self.client)
	
	def handle_event(self, event):
		if self.client == None:
			if event.type == pygame.KEYDOWN:
				self.show_error = False
				try:
					letter = event.unicode.encode('ASCII')
					if letter in allowed_letters and len(self.server_hostname) < 64:
						self.server_hostname += letter
						self._update_labels()
				except:	
					pass
				if event.key == pygame.K_BACKSPACE:
					self.server_hostname = self.server_hostname[:-1]
					self._update_labels()
				if event.key == pygame.K_RETURN:
					self._connect()
				if event.key == pygame.K_ESCAPE:
					self._quit()
			if event.type == pygame.MOUSEBUTTONDOWN:
				button_pos = [(800-self.button_sprite.size[0])/2,220]
				button_size = self.button_sprite.size
				if event.pos[0] < button_pos[0] or event.pos[0] > button_pos[0]+button_size[0]:
					return
				if event.pos[1] < button_pos[1] or event.pos[1] > button_pos[1]+button_size[1]:
					return
				self._connect()
