import pygame
from OpenGL import GL
import ode
import math

import screen
import ship
import rope

import pdb

key_mapping = {
	pygame.K_LEFT: 0,
	pygame.K_DOWN: 1,
	pygame.K_RIGHT: 2,
	pygame.K_UP: 3,
	}

class Level (screen.Screen):
	def __load_level(self, lvlname):
		fh = open("data\\%s.lvl" % lvlname,"rt")
		llist = fh.readlines()
		fh.close()
		for line in llist:
			if "" == line.strip():
					continue
			tup = line.strip().split(" ")
			tup = tuple(map(lambda x:float(x),tup))
			lens = (abs(tup[2]-tup[0]),abs(tup[3]-tup[1]),1)
			wall = ode.GeomBox(self.ode_space,lengths=lens)
			wall.coords = tup
			wall.setPosition(((tup[0] + tup[2])/2.0,(tup[1] + tup[3])/2.0,0))
			self.ode_walls.append(wall)
                        
	def __init__(self, lvlname, network_object):
		super(Level,self).__init__()
		lvlname = network_object.handle_level_name(lvlname)
		
		self.ships = []
		self.ships.append(ship.Ship([0,+2.9],+math.pi/2,[+8,0]))
		self.ships.append(ship.Ship([0,-2.9],-math.pi/2,[-8,0]))
		
		self.ode_walls = []
		self.ode_space = ode.Space()
		self.__load_level(lvlname)
		self.network_object = network_object
		self.client_id = self.network_object.get_client_id()
	
	def draw(self):
		GL.glColor3f(0.5,0.5,0.5)
		GL.glScalef(32,32,1) #minimum zoom level (ship of size 1 will be 32 pixels on screen)
		for wall in self.ode_walls:
			GL.glRectf(*wall.coords)
		
		rope.draw(self.ships[0], self.ships[1])
		for ship in self.ships:
			ship.draw()
	
	def ode_near(self, args, geom1, geom2):
		contacts = ode.collide(geom1,geom2)
		for c in contacts:
			print "COLLISION!"
	
	def do_frame(self, time_passed):
		if (self.network_object != None):
			try:
				self.network_object.do_frame(self, time_passed)
			except Exception, e:
				print "Error: %s\n" % e
				print "Removing network object\n"
				self.network_object = None
		for ship in self.ships:
			ship.do_frame(time_passed)
			ship_geom = ode.GeomBox(self.ode_space,(1.0/32.0,1.0/32,1))
			ship_geom.setPosition((ship.position[0],ship.position[1],0))
			#print ship_geom.getPosition()
			self.ode_space.collide(None,self.ode_near)
			self.ode_space.remove(ship_geom)
		
		rope.pull(self.ships[0], self.ships[1], time_passed)
	
	def handle_event(self, event):
		#KEYDOWN	unicode, key, mod
		#KEYUP		key, mod
		if event.type == pygame.KEYDOWN or event.type == pygame.KEYUP:
			if key_mapping.has_key(event.key):
				mapping = key_mapping[event.key]
				self.ships[self.client_id].keys_pressed[mapping] = (event.type == pygame.KEYDOWN)
				if self.network_object != None:
					self.network_object.send_updates(self.client_id, mapping, event.type == pygame.KEYDOWN)
	
