import socket
import select

import network

class Server:
	def __init__(self):
		self.listening_socket = socket.socket()
		self.listening_socket.bind(("0.0.0.0", network.default_server_port))
		self.listening_socket.listen(3) #maximum 3 connections are cached (usually they will just
		self.listening_socket.setblocking(False)
		
		self.client_sockets = []
		
		self.time_since_last_update = 0
		self.next_ship_to_update = 0
	
	def get_client_id(self):
		return 0
	
	def check_new_client(self):
		try:
			new_sock,address = self.listening_socket.accept()
			network.send_client_information(new_sock, len(self.client_sockets)+1)
			self.client_sockets.append(new_sock)
		except socket.error, e:
			pass
	
	def stop_listening(self):
		self.listening_socket.close()
	
	def send_updates(self, ship_index, key_index, is_down):
		for sock in self.client_sockets:
			network.send_key_information(sock, ship_index, key_index, is_down)
	
	def send_start_game(self):
		for sock in self.client_sockets:
			network.send_start_game(sock)
	
	def handle_level_name(self, level_name):
		dead_clients = []
		for sock in self.client_sockets:
			try:
				network.send_string(sock, level_name)
			except:
				dead_clients.append(self.client_sockets)
		for dead_client in dead_clients:
			self.client_sockets.remove(dead_client)
		return level_name
	
	def do_frame(self, level, time_passed):
		#first, check for updates from the clients
		while (True):
			ready_to_recv = []
			if self.client_sockets != []:
				ready_to_recv = select.select(self.client_sockets,[],[],0)[0]
			if ready_to_recv == []:
				break
			#pass on each socket once(one command) every loop, until all processing is done
			for client_socket in ready_to_recv:
				try:
					command = client_socket.recv(1)
					if command == network.KEY_UPDATE_CODE:
						ship_index, key_index, is_down = network.get_key_information(client_socket)
						level.ships[ship_index].keys_pressed[key_index] = is_down
						for other_client in filter(lambda s: s!= client_socket, self.client_sockets):
							network.send_key_information(other_client, ship_index, key_index, is_down)
					elif command == 'Q':
						pass #end game?
					else:
						raise Exception("Unrecognized protocol")
				except Exception, e:
					#when a client does a problem, kick him out
					print "Error handling a client message: %s\n" % e
					client_socket.close()
					self.client_sockets.remove(client_socket)
		
		#send regular updates to clients about ship position and other information
		self.time_since_last_update += time_passed
		if self.time_since_last_update > 0.3:
			self.time_since_last_update = 0
			
			ship_to_update = level.ships[self.next_ship_to_update]
			for client_socket in self.client_sockets:
				network.send_ship_information(client_socket, self.next_ship_to_update, ship_to_update)
			
			self.next_ship_to_update += 1
			self.next_ship_to_update %= len(level.ships)
	
	def __del__(self):
		for sock in self.client_sockets:
			sock.close()
		try:
			self.listening_socket.close()
		except:
			pass
	